Stan Math Library: stan/math/prim/scal/prob/hypergeometric_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

hypergeometric_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_HYPERGEOMETRIC_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_HYPERGEOMETRIC_LOG_HPP
3
4 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
5 #include <stan/math/prim/scal/err/check_bounded.hpp>
6 #include <stan/math/prim/scal/err/check_finite.hpp>
7 #include <stan/math/prim/scal/err/check_greater.hpp>
8 #include <stan/math/prim/scal/err/check_positive.hpp>
9 #include <stan/math/prim/scal/fun/binomial_coefficient_log.hpp>
10 #include <stan/math/prim/scal/meta/length.hpp>
11 #include <stan/math/prim/scal/meta/VectorView.hpp>
12 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
13 #include <stan/math/prim/scal/meta/return_type.hpp>
14 #include <stan/math/prim/scal/meta/include_summand.hpp>
15 #include <stan/math/prim/scal/fun/constants.hpp>
16 #include <boost/math/distributions.hpp>
17
18 namespace stan {
19
20 namespace math {
21
22 // Hypergeometric(n|N, a, b) [0 <= n <= a; 0 <= N-n <= b; 0 <= N <= a+b]
23 // n: #white balls drawn; N: #balls drawn;
24 // a: #white balls; b: #black balls
25 template <bool propto,
26 typename T_n, typename T_N,
27 typename T_a, typename T_b>
28 double
29 hypergeometric_log(const T_n& n, const T_N& N,
30 const T_a& a, const T_b& b) {
31 static const char* function("stan::math::hypergeometric_log");
32
33 using stan::math::check_finite;
34 using stan::math::check_bounded;
35 using stan::math::check_greater;
36 using stan::math::check_consistent_sizes;
37 using stan::math::include_summand;
38
39 // check if any vectors are zero length
40 if (!(stan::length(n)
41 && stan::length(N)
42 && stan::length(a)
43 && stan::length(b)))
44 return 0.0;
45
46
47 VectorView<const T_n> n_vec(n);
48 VectorView<const T_N> N_vec(N);
49 VectorView<const T_a> a_vec(a);
50 VectorView<const T_b> b_vec(b);
51 size_t size = max_size(n, N, a, b);
52
53 double logp(0.0);
54 check_bounded(function, "Successes variable", n, 0, a);
55 check_greater(function, "Draws parameter", N, n);
56 for (size_t i = 0; i < size; i++) {
57 check_bounded(function, "Draws parameter minus successes variable",
58 N_vec[i]-n_vec[i], 0, b_vec[i]);
59 check_bounded(function, "Draws parameter", N_vec[i], 0,
60 a_vec[i]+b_vec[i]);
61 }
62 check_consistent_sizes(function,
63 "Successes variable", n,
64 "Draws parameter", N,
65 "Successes in population parameter", a,
66 "Failures in population parameter", b);
67
68 // check if no variables are involved and prop-to
69 if (!include_summand<propto>::value)
70 return 0.0;
71
72
73 for (size_t i = 0; i < size; i++)
74 logp += math::binomial_coefficient_log(a_vec[i], n_vec[i])
75 + math::binomial_coefficient_log(b_vec[i], N_vec[i]-n_vec[i])
76 - math::binomial_coefficient_log(a_vec[i]+b_vec[i], N_vec[i]);
77 return logp;
78 }
79
80 template <typename T_n,
81 typename T_N,
82 typename T_a,
83 typename T_b>
84 inline
85 double
86 hypergeometric_log(const T_n& n,
87 const T_N& N,
88 const T_a& a,
89 const T_b& b) {
90 return hypergeometric_log<false>(n, N, a, b);
91 }
92 }
93 }
94 #endif
stan::math::binomial_coefficient_log
fvar< T > binomial_coefficient_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: binomial_coefficient_log.hpp:16
stan::math::hypergeometric_log
double hypergeometric_log(const T_n &n, const T_N &N, const T_a &a, const T_b &b)
Definition: hypergeometric_log.hpp:29
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::check_bounded
bool check_bounded(const char *function, const char *name, const T_y &y, const T_low &low, const T_high &high)
Return true if the value is between the low and high values, inclusively.
Definition: check_bounded.hpp:95
binomial_coefficient_log.hpp
return_type.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
include_summand.hpp
check_greater.hpp
VectorView.hpp
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
check_bounded.hpp
length.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_greater
bool check_greater(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is strictly greater than low.
Definition: check_greater.hpp:84

[ Stan Home Page ] © 2011–2016, Stan Development Team.